@@ -73,7 +73,8 @@ _gedit_cmd_file_new (GtkAction   *action,
 }
 
 static GeditTab *
-get_tab_from_file (GList *docs, GFile *file)
+get_tab_from_file (GList *docs,
+		   GFile *file)
 {
 	GeditTab *tab = NULL;
 
@@ -104,7 +105,8 @@ get_tab_from_file (GList *docs, GFile *file)
 }
 
 static gboolean
-is_duplicated_file (GSList *files, GFile *file)
+is_duplicated_file (GSList *files,
+		    GFile  *file)
 {
 	while (files != NULL)
 	{
@@ -118,19 +120,21 @@ is_duplicated_file (GSList *files, GFile *file)
 }
 
 /* File loading */
-static gint
+static GSList *
 load_file_list (GeditWindow         *window,
-		GSList              *files,
+		const GSList        *files,
 		const GeditEncoding *encoding,
 		gint                 line_pos,
+		gint                 column_pos,
 		gboolean             create)
 {
 	GeditTab      *tab;
-	gint           loaded_files = 0; /* Number of files to load */
+	GSList        *loaded_files = NULL; /* Number of files to load */
 	gboolean       jump_to = TRUE; /* Whether to jump to the new tab */
 	GList         *win_docs;
 	GSList        *files_to_load = NULL;
-	GSList        *l;
+	const GSList  *l;
+	gint num_loaded_files = 0;
 
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -147,24 +151,38 @@ load_file_list (GeditWindow         *window,
 			{
 				if (l == files)
 				{
+					GeditDocument *doc;
+
 					gedit_window_set_active_tab (window, tab);
 					jump_to = FALSE;
+					doc = gedit_tab_get_document (tab);
 
 					if (line_pos > 0)
 					{
-						GeditDocument *doc;
-						GeditView *view;
-
-						doc = gedit_tab_get_document (tab);
-						view = gedit_tab_get_view (tab);
-
 						/* document counts lines starting from 0 */
-						gedit_document_goto_line (doc, line_pos - 1);
-						gedit_view_scroll_to_cursor (view);
+						if (column_pos > 0)
+						{
+							GtkTextIter iter;
+
+							gtk_text_buffer_get_iter_at_line_offset (GTK_TEXT_BUFFER (doc),
+							                                         &iter,
+							                                         line_pos - 1,
+							                                         column_pos - 1);
+
+							gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc), &iter);
+						}
+						else
+						{
+							gedit_document_goto_line (doc, line_pos - 1);
+						}
+
+						gedit_view_scroll_to_cursor (gedit_tab_get_view (tab));
 					}
 				}
 
-				++loaded_files;
+				++num_loaded_files;
+				loaded_files = g_slist_prepend (loaded_files,
+				                                gedit_tab_get_document (tab));
 			}
 			else
 			{
@@ -177,8 +195,10 @@ load_file_list (GeditWindow         *window,
 	g_list_free (win_docs);
 
 	if (files_to_load == NULL)
-		return loaded_files;
-	
+	{
+		return g_slist_reverse (loaded_files);
+	}
+
 	files_to_load = g_slist_reverse (files_to_load);
 	l = files_to_load;
 
@@ -192,50 +212,52 @@ load_file_list (GeditWindow         *window,
 		if (gedit_document_is_untouched (doc) &&
 		    (gedit_tab_get_state (tab) == GEDIT_TAB_STATE_NORMAL))
 		{
-			gchar *uri;
-
-			// FIXME: pass the GFile to tab when api is there
-			uri = g_file_get_uri (l->data);
 			_gedit_tab_load (tab,
-					 uri,
+					 l->data,
 					 encoding,
 					 line_pos,
+					 column_pos,
 					 create);
-			g_free (uri);
+			
+			/* make sure the view has focus */
+			gtk_widget_grab_focus (GTK_WIDGET (gedit_tab_get_view (tab)));
 
 			l = g_slist_next (l);
 			jump_to = FALSE;
 
-			++loaded_files;
+			++num_loaded_files;
+			loaded_files = g_slist_prepend (loaded_files,
+			                                gedit_tab_get_document (tab));
 		}
 	}
 
 	while (l != NULL)
 	{
-		gchar *uri;
-
 		g_return_val_if_fail (l->data != NULL, 0);
 
-		// FIXME: pass the GFile to tab when api is there
-		uri = g_file_get_uri (l->data);
-		tab = gedit_window_create_tab_from_uri (window,
-							uri,
-							encoding,
-							line_pos,
-							create,
-							jump_to);
-		g_free (uri);
+		tab = gedit_window_create_tab_from_location (window,
+							     l->data,
+							     encoding,
+							     line_pos,
+							     column_pos,
+							     create,
+							     jump_to);
 
 		if (tab != NULL)
 		{
 			jump_to = FALSE;
-			++loaded_files;
+
+			++num_loaded_files;
+			loaded_files = g_slist_prepend (loaded_files,
+			                                gedit_tab_get_document (tab));
 		}
 
 		l = g_slist_next (l);
 	}
 
-	if (loaded_files == 1)
+	loaded_files = g_slist_reverse (loaded_files);
+
+	if (num_loaded_files == 1)
 	{
 		GeditDocument *doc;
 		gchar *uri_for_display;
@@ -258,8 +280,8 @@ load_file_list (GeditWindow         *window,
 					       window->priv->generic_message_cid,
 					       ngettext("Loading %d file\342\200\246",
 							"Loading %d files\342\200\246",
-							loaded_files),
-					       loaded_files);
+							num_loaded_files),
+					       num_loaded_files);
 	}
 
 	/* Free uris_to_load. Note that l points to the first element of uris_to_load */
@@ -268,100 +290,70 @@ load_file_list (GeditWindow         *window,
 	return loaded_files;
 }
 
-
-// FIXME: we should expose API with GFile and just make the uri
-// variants backward compat wrappers
-
-static gint
-load_uri_list (GeditWindow         *window,
-	       const GSList        *uris,
-	       const GeditEncoding *encoding,
-	       gint                 line_pos,
-	       gboolean             create)
-{
-	GSList *files = NULL;
-	const GSList *u;
-	gint ret;
-
-	for (u = uris; u != NULL; u = u->next)
-	{
-		gchar *uri = u->data;
-
-		if (gedit_utils_is_valid_uri (uri))
-			files = g_slist_prepend (files, g_file_new_for_uri (uri));
-		else
-			g_warning ("invalid uri: %s", uri);
-	}
-	files = g_slist_reverse (files);
-
-	ret = load_file_list (window, files, encoding, line_pos, create);
-
-	g_slist_foreach (files, (GFunc) g_object_unref, NULL);
-	g_slist_free (files);
-
-	return ret;
-}
-
 /**
  * gedit_commands_load_uri:
+ * @window:
+ * @location:
+ * @encoding: (allow-none):
+ * @line_pos:
+ * @column_pos:
  *
- * Do nothing if URI does not exist
+ * Ignore non-existing locations
+ *
+ * Returns: (transfer container):
  */
 void
-gedit_commands_load_uri (GeditWindow         *window,
-			 const gchar         *uri,
-			 const GeditEncoding *encoding,
-			 gint                 line_pos)
+gedit_commands_load_location (GeditWindow         *window,
+			      GFile               *location,
+			      const GeditEncoding *encoding,
+			      gint                 line_pos,
+			      gint                 column_pos)
 {
-	GSList *uris = NULL;
+	GSList *locations = NULL;
+	gchar *uri;
+	GSList *ret;
 
 	g_return_if_fail (GEDIT_IS_WINDOW (window));
-	g_return_if_fail (uri != NULL);
-	g_return_if_fail (gedit_utils_is_valid_uri (uri));
+	g_return_if_fail (G_IS_FILE (location));
+	g_return_if_fail (gedit_utils_is_valid_location (location));
 
+	uri = g_file_get_uri (location);
 	gedit_debug_message (DEBUG_COMMANDS, "Loading URI '%s'", uri);
+	g_free (uri);
 
-	uris = g_slist_prepend (uris, (gchar *)uri);
+	locations = g_slist_prepend (locations, location);
 
-	load_uri_list (window, uris, encoding, line_pos, FALSE);
+	ret = load_file_list (window, locations, encoding, line_pos, column_pos, FALSE);
+	g_slist_free (ret);
 
-	g_slist_free (uris);
+	g_slist_free (locations);
 }
 
 /**
  * gedit_commands_load_uris:
+ * @window:
+ * @locations: (element-type Glib.File) (transfer none):
+ * @encoding: (allow-none):
+ * @line_pos:
+ * @column_pos:
  *
- * Ignore non-existing URIs 
+ * Ignore non-existing locations
+ *
+ * Returns: (transfer container):
  */
-gint
-gedit_commands_load_uris (GeditWindow         *window,
-			  const GSList        *uris,
-			  const GeditEncoding *encoding,
-			  gint                 line_pos)
+GSList *
+gedit_commands_load_locations (GeditWindow         *window,
+			       const GSList        *locations,
+			       const GeditEncoding *encoding,
+			       gint                 line_pos,
+			       gint                 column_pos)
 {
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), 0);
-	g_return_val_if_fail ((uris != NULL) && (uris->data != NULL), 0);
+	g_return_val_if_fail ((locations != NULL) && (locations->data != NULL), 0);
 
 	gedit_debug (DEBUG_COMMANDS);
 
-	return load_uri_list (window, uris, encoding, line_pos, FALSE);
-}
-
-/*
- * This should become public once we convert all api to GFile:
- */
-static gint
-gedit_commands_load_files (GeditWindow         *window,
-			   GSList              *files,
-			   const GeditEncoding *encoding,
-			   gint                 line_pos)
-{
-	g_return_val_if_fail (GEDIT_IS_WINDOW (window), 0);
-	g_return_val_if_fail ((files != NULL) && (files->data != NULL), 0);
-
-	gedit_debug (DEBUG_COMMANDS);
-
-	return load_file_list (window, files, encoding, line_pos, FALSE);
+	return load_file_list (window, locations, encoding, line_pos, column_pos, FALSE);
 }
 
 /*
@@ -369,15 +361,16 @@ gedit_commands_load_files (GeditWindow         *window,
  * first doc. Beside specifying a not existing uri creates a
  * titled document.
  */
-gint
+GSList *
 _gedit_cmd_load_files_from_prompt (GeditWindow         *window,
 				   GSList              *files,
 				   const GeditEncoding *encoding,
-				   gint                 line_pos)
+				   gint                 line_pos,
+				   gint                 column_pos)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
-	return load_file_list (window, files, encoding, line_pos, TRUE);
+	return load_file_list (window, files, encoding, line_pos, column_pos, TRUE);
 }
 
 static void
@@ -398,6 +391,7 @@ open_dialog_response_cb (GeditFileChooserDialog *dialog,
 {
 	GSList *files;
 	const GeditEncoding *encoding;
+	GSList *loaded;
 
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -418,10 +412,13 @@ open_dialog_response_cb (GeditFileChooserDialog *dialog,
 	/* Remember the folder we navigated to */
 	 _gedit_window_set_default_location (window, files->data);
 
-	gedit_commands_load_files (window,
-				   files,
-				   encoding,
-				   0);
+	loaded = gedit_commands_load_locations (window,
+	                                        files,
+	                                        encoding,
+	                                        0,
+	                                        0);
+
+	g_slist_free (loaded);
 
 	g_slist_foreach (files, (GFunc) g_object_unref, NULL);
 	g_slist_free (files);
@@ -505,7 +502,8 @@ _gedit_cmd_file_open (GtkAction   *action,
 }
 
 /* File saving */
-static void file_save_as (GeditTab *tab, GeditWindow *window);
+static void file_save_as (GeditTab    *tab,
+			  GeditWindow *window);
 
 static gboolean
 is_read_only (GFile *location)
@@ -539,7 +537,8 @@ is_read_only (GFile *location)
 /* FIXME: modify this dialog to be similar to the one provided by gtk+ for
  * already existing files - Paolo (Oct. 11, 2005) */
 static gboolean
-replace_read_only_file (GtkWindow *parent, GFile *file)
+replace_read_only_file (GtkWindow *parent,
+			GFile     *file)
 {
 	GtkWidget *dialog;
 	gint ret;
@@ -590,17 +589,106 @@ replace_read_only_file (GtkWindow *parent, GFile *file)
 	return (ret == GTK_RESPONSE_YES);
 }
 
+static gboolean
+change_compression (GtkWindow *parent,
+                    GFile     *file,
+                    gboolean   compressed)
+{
+	GtkWidget *dialog;
+	gint ret;
+	gchar *parse_name;
+	gchar *name_for_display;
+	const gchar *primary_message;
+	const gchar *secondary_message;
+	const gchar *button_label;
+
+	gedit_debug (DEBUG_COMMANDS);
+
+	parse_name = g_file_get_parse_name (file);
+
+	/* Truncate the name so it doesn't get insanely wide. Note that even
+	 * though the dialog uses wrapped text, if the name doesn't contain
+	 * white space then the text-wrapping code is too stupid to wrap it.
+	 */
+	name_for_display = gedit_utils_str_middle_truncate (parse_name, 50);
+	g_free (parse_name);
+
+	if (compressed)
+	{
+		primary_message = _("Save the file using compression?");
+		secondary_message = _("The file \"%s\" was previously saved as plain "
+		                      "text and will now be saved using compression.");
+		button_label = _("_Save Using Compression");
+	}
+	else
+	{
+		primary_message = _("Save the file as plain text?");
+		secondary_message = _("The file \"%s\" was previously saved "
+		                      "using compression and will now be saved as plain text.");
+		button_label = _("_Save As Plain Text");
+	}
+
+	dialog = gtk_message_dialog_new (parent,
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_NONE,
+					 "%s",
+					 primary_message);
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  secondary_message,
+						  name_for_display);
+
+	g_free (name_for_display);
+
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       GTK_STOCK_CANCEL,
+			       GTK_RESPONSE_CANCEL);
+
+	gedit_dialog_add_button (GTK_DIALOG (dialog),
+				 button_label,
+			  	 GTK_STOCK_SAVE_AS,
+			  	 GTK_RESPONSE_YES);
+
+	gtk_dialog_set_default_response	(GTK_DIALOG (dialog),
+					 GTK_RESPONSE_CANCEL);
+
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+	ret = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+
+	return (ret == GTK_RESPONSE_YES);
+}
+
+static GeditDocumentCompressionType
+get_compression_type_from_file (GFile *file)
+{
+	gchar *name;
+	gchar *content_type;
+	GeditDocumentCompressionType type;
+
+	name = g_file_get_basename (file);
+	content_type = g_content_type_guess (name, NULL, 0, NULL);
+
+	type = gedit_utils_get_compression_type_from_content_type (content_type);
+
+	g_free (name);
+	g_free (content_type);
+
+	return type;
+}
+
 static void
 save_dialog_response_cb (GeditFileChooserDialog *dialog,
                          gint                    response_id,
                          GeditWindow            *window)
 {
 	GFile *file;
-	const GeditEncoding *encoding;
 	GeditTab *tab;
 	gpointer data;
 	GSList *tabs_to_save_as;
-	GeditDocumentNewlineType newline_type;
 
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -617,16 +705,36 @@ save_dialog_response_cb (GeditFileChooserDialog *dialog,
 	file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
 	g_return_if_fail (file != NULL);
 
-	encoding = gedit_file_chooser_dialog_get_encoding (dialog);
-	newline_type = gedit_file_chooser_dialog_get_newline_type (dialog);
-
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-
 	if (tab != NULL)
 	{
 		GeditDocument *doc;
 		gchar *parse_name;
-		gchar *uri;
+		GeditDocumentNewlineType newline_type;
+		GeditDocumentCompressionType compression_type;
+		GeditDocumentCompressionType current_compression_type;
+		const GeditEncoding *encoding;
+
+		doc = gedit_tab_get_document (tab);
+		compression_type = get_compression_type_from_file (file);
+		current_compression_type = gedit_document_get_compression_type (doc);
+
+		if ((compression_type == GEDIT_DOCUMENT_COMPRESSION_TYPE_NONE) !=
+		    (current_compression_type == GEDIT_DOCUMENT_COMPRESSION_TYPE_NONE))
+		{
+			if (!change_compression (GTK_WINDOW (dialog),
+			                         file,
+			                         compression_type != GEDIT_DOCUMENT_COMPRESSION_TYPE_NONE))
+			{
+				gtk_widget_destroy (GTK_WIDGET (dialog));
+
+				goto save_next_tab;
+			}
+		}
+
+		encoding = gedit_file_chooser_dialog_get_encoding (dialog);
+		newline_type = gedit_file_chooser_dialog_get_newline_type (dialog);
+
+		gtk_widget_destroy (GTK_WIDGET (dialog));
 
 		doc = gedit_tab_get_document (tab);
 		g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
@@ -644,10 +752,7 @@ save_dialog_response_cb (GeditFileChooserDialog *dialog,
 		 * even if the saving fails... */
 		 _gedit_window_set_default_location (window, file);
 
-		// FIXME: pass the GFile to tab when api is there
-		uri = g_file_get_uri (file);
-		_gedit_tab_save_as (tab, uri, encoding, newline_type);
-		g_free (uri);
+		_gedit_tab_save_as (tab, file, encoding, newline_type, compression_type);
 	}
 
 	g_object_unref (file);
@@ -743,7 +848,7 @@ file_save_as (GeditTab    *tab,
 
 	gedit_debug (DEBUG_COMMANDS);
 
-	save_dialog = gedit_file_chooser_dialog_new (_("Save As\342\200\246"),
+	save_dialog = gedit_file_chooser_dialog_new (_("Save As"),
 						     GTK_WINDOW (window),
 						     GTK_FILE_CHOOSER_ACTION_SAVE,
 						     NULL,
@@ -779,7 +884,6 @@ file_save_as (GeditTab    *tab,
 		g_object_unref (file);
 	}
 
-
 	if (!uri_set)
 	{
 		GFile *default_path;
@@ -868,7 +972,7 @@ file_save (GeditTab    *tab,
 
 void
 _gedit_cmd_file_save (GtkAction   *action,
-		     GeditWindow *window)
+		      GeditWindow *window)
 {
 	GeditTab *tab;
 
@@ -883,7 +987,7 @@ _gedit_cmd_file_save (GtkAction   *action,
 
 void
 _gedit_cmd_file_save_as (GtkAction   *action,
-			GeditWindow *window)
+			 GeditWindow *window)
 {
 	GeditTab *tab;
 
@@ -1037,7 +1141,7 @@ gedit_commands_save_all_documents (GeditWindow *window)
 
 void
 _gedit_cmd_file_save_all (GtkAction   *action,
-			 GeditWindow *window)
+			  GeditWindow *window)
 {
 	gedit_commands_save_all_documents (window);
 }
@@ -1225,7 +1329,7 @@ revert_dialog (GeditWindow   *window,
 
 void
 _gedit_cmd_file_revert (GtkAction   *action,
-		       GeditWindow *window)
+		        GeditWindow *window)
 {
 	GeditTab       *tab;
 	GeditDocument  *doc;
@@ -1238,9 +1342,11 @@ _gedit_cmd_file_revert (GtkAction   *action,
 	g_return_if_fail (tab != NULL);
 
 	/* If we are already displaying a notification
-	 * reverting will drop local modifications, do
+	 * reverting will drop local modifications or
+	 * if the document has not been modified, do
 	 * not bug the user further */
-	if (gedit_tab_get_state (tab) == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
+	if (gedit_tab_get_state (tab) == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION ||
+	    _gedit_tab_can_close (tab))
 	{
 		do_revert (window, tab);
 		return;
@@ -1379,19 +1485,18 @@ static void
 save_and_close_all_documents (const GList  *docs,
 			      GeditWindow  *window)
 {
-	GList  *tabs;
-	GList  *l;
+	GList *tabs;
+	GList *l;
 	GSList *sl;
 	GSList *tabs_to_save_as;
 	GSList *tabs_to_save_and_close;
-	GList  *tabs_to_close;
+	GList *tabs_to_close;
 
 	gedit_debug (DEBUG_COMMANDS);
 
 	g_return_if_fail (!(gedit_window_get_state (window) & GEDIT_WINDOW_STATE_PRINTING));
 
-	tabs = gtk_container_get_children (
-			GTK_CONTAINER (_gedit_window_get_notebook (window)));
+	tabs = _gedit_window_get_all_tabs (window);
 
 	tabs_to_save_as = NULL;
 	tabs_to_save_and_close = NULL;
@@ -1597,7 +1702,9 @@ close_confirmation_dialog_response_handler (GeditCloseConfirmationDialog *dlg,
 					return;
 				}
 				else
+				{
 					g_return_if_reached ();
+				}
 			}
 			else
 			{
@@ -1640,6 +1747,7 @@ close_confirmation_dialog_response_handler (GeditCloseConfirmationDialog *dlg,
 			}
 
 			break;
+
 		default: /* Do not close */
 
 			/* Reset is_quitting flag */
@@ -1673,10 +1781,9 @@ tab_can_close (GeditTab  *tab,
 	{
 		GtkWidget     *dlg;
 
-		dlg = gedit_close_confirmation_dialog_new_single (
-						window,
-						doc,
-						FALSE);
+		dlg = gedit_close_confirmation_dialog_new_single (window,
+								  doc,
+								  FALSE);
 
 		g_signal_connect (dlg,
 				  "response",
@@ -1724,7 +1831,7 @@ _gedit_cmd_file_close_tab (GeditTab    *tab,
 
 void
 _gedit_cmd_file_close (GtkAction   *action,
-		      GeditWindow *window)
+		       GeditWindow *window)
 {
 	GeditTab *active_tab;
 
@@ -1817,7 +1924,7 @@ file_close_all (GeditWindow *window,
 
 void
 _gedit_cmd_file_close_all (GtkAction   *action,
-			  GeditWindow *window)
+			   GeditWindow *window)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -1857,6 +1964,8 @@ quit_all ()
 			file_close_all (window, TRUE);
 		}
 	}
+
+	g_list_free (windows);
 }
 #endif
 
@@ -1881,3 +1990,5 @@ _gedit_cmd_file_quit (GtkAction   *action,
 
 	file_close_all (window, TRUE);
 }
+
+/* ex:ts=8:noet: */
