@@ -91,6 +91,9 @@ typedef struct _GeditTabClass GeditTabClass;
 struct _GeditTabClass 
 {
 	GtkVBoxClass parent_class;
+
+	void (* drop_uris)	(GeditView *view,
+				 gchar    **uri_list);
 };
 
 /*
@@ -142,7 +145,7 @@ GtkWidget	*_gedit_tab_new_from_stream	(GInputStream        *stream,
 						 gint                 column_pos);
 
 gchar 		*_gedit_tab_get_name		(GeditTab            *tab);
-gchar 		*_gedit_tab_get_tooltips	(GeditTab            *tab);
+gchar 		*_gedit_tab_get_tooltip		(GeditTab            *tab);
 GdkPixbuf 	*_gedit_tab_get_icon		(GeditTab            *tab);
 void		 _gedit_tab_load		(GeditTab            *tab,
 						 GFile               *location,
@@ -172,9 +175,6 @@ void		 _gedit_tab_mark_for_closing	(GeditTab	     *tab);
 
 gboolean	 _gedit_tab_can_close		(GeditTab	     *tab);
 
-#if !GTK_CHECK_VERSION (2, 17, 4)
-void		 _gedit_tab_page_setup		(GeditTab            *tab);
-#endif
 
 G_END_DECLS
 
