@@ -48,7 +48,6 @@
 #include "gedit-utils.h"
 
 #include "gedit-document.h"
-#include "gedit-prefs-manager.h"
 #include "gedit-debug.h"
 
 /* For the workspace/viewport stuff */
@@ -59,55 +58,27 @@
 #include <X11/Xatom.h>
 #endif
 
+#ifdef G_OS_UNIX
+#include <unistd.h>
+#include <sys/stat.h>
+#endif
+
 #define STDIN_DELAY_MICROSECONDS 100000
 
-/* Returns true if uri is a file: uri and is not a chained uri */
+/**
+ * gedit_utils_location_has_file_scheme:
+ *
+ * Returns: %TRUE if @location is a file: uri and is not a chained uri */
 gboolean
-gedit_utils_uri_has_file_scheme (const gchar *uri)
+gedit_utils_location_has_file_scheme (GFile *location)
 {
-	GFile *gfile;
-	gboolean res;
-
-	gfile = g_file_new_for_uri (uri);
-	res = g_file_has_uri_scheme (gfile, "file");
-	
-	g_object_unref (gfile);
-	return res;
-}
-
-/* FIXME: we should check for chained URIs */
-gboolean
-gedit_utils_uri_has_writable_scheme (const gchar *uri)
-{
-	GFile *gfile;
-	gchar *scheme;
-	GSList *writable_schemes;
-	gboolean res;
-
-	gfile = g_file_new_for_uri (uri);
-	scheme = g_file_get_uri_scheme (gfile);
-
-	g_return_val_if_fail (scheme != NULL, FALSE);
-
-	g_object_unref (gfile);
-
-	writable_schemes = gedit_prefs_manager_get_writable_vfs_schemes ();
-
-	/* CHECK: should we use g_ascii_strcasecmp? - Paolo (Nov 6, 2005) */
-	res = (g_slist_find_custom (writable_schemes,
-				    scheme,
-				    (GCompareFunc)strcmp) != NULL);
-
-	g_slist_foreach (writable_schemes, (GFunc)g_free, NULL);
-	g_slist_free (writable_schemes);
-
-	g_free (scheme);
-
-	return res;
+	return g_file_has_uri_scheme (location, "file");
 }
 
 static void
-widget_get_origin (GtkWidget *widget, gint *x, gint *y)
+widget_get_origin (GtkWidget *widget,
+		   gint      *x,
+		   gint      *y)
 
 {
 	GdkWindow *window;
@@ -125,22 +96,25 @@ gedit_utils_menu_position_under_widget (GtkMenu  *menu,
 {
 	GtkWidget *widget;
 	GtkRequisition requisition;
+	GtkAllocation allocation;
 
 	widget = GTK_WIDGET (user_data);
 	widget_get_origin (widget, x, y);
 
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
 
+	gtk_widget_get_allocation (widget, &allocation);
+
 	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
 	{
-		*x += widget->allocation.x + widget->allocation.width - requisition.width;
+		*x += allocation.x + allocation.width - requisition.width;
 	}
 	else
 	{
-		*x += widget->allocation.x;
+		*x += allocation.x;
 	}
 
-	*y += widget->allocation.y + widget->allocation.height;
+	*y += allocation.y + allocation.height;
 
 	*push_in = TRUE;
 }
@@ -195,16 +169,6 @@ gedit_utils_menu_position_under_tree_view (GtkMenu  *menu,
 	}
 }
 
-/* FIXME: remove this with gtk 2.12, it has gdk_color_to_string */
-gchar * 
-gedit_gdk_color_to_string (GdkColor color)
-{
-	return g_strdup_printf ("#%04x%04x%04x",
-				color.red, 
-				color.green,
-				color.blue);
-}
-
 GtkWidget *
 gedit_gtk_button_new_with_stock_icon (const gchar *label,
 				      const gchar *stock_id)
@@ -216,7 +180,7 @@ gedit_gtk_button_new_with_stock_icon (const gchar *label,
 			      gtk_image_new_from_stock (stock_id,
 							GTK_ICON_SIZE_BUTTON));
 
-        return button;
+	return button;
 }
 
 GtkWidget *
@@ -234,7 +198,7 @@ gedit_dialog_add_button (GtkDialog   *dialog,
 	button = gedit_gtk_button_new_with_stock_icon (text, stock_id);
 	g_return_val_if_fail (button != NULL, NULL);
 
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (button, TRUE);
 
 	gtk_widget_show (button);
 
@@ -247,7 +211,10 @@ gedit_dialog_add_button (GtkDialog   *dialog,
  * n: len of the string in bytes
  */
 gboolean 
-g_utf8_caselessnmatch (const char *s1, const char *s2, gssize n1, gssize n2)
+g_utf8_caselessnmatch (const char *s1,
+		       const char *s2,
+		       gssize      n1,
+		       gssize      n2)
 {
 	gchar *casefold;
 	gchar *normalized_s1;
@@ -285,15 +252,16 @@ finally_2:
 }
 
 /**
- * gedit_utils_set_atk_name_description
- * @widget : The Gtk widget for which name/description to be set
- * @name : Atk name string
- * @description : Atk description string
- * Description : This function sets up name and description
+ * gedit_utils_set_atk_name_description:
+ * @widget: The Gtk widget for which name/description to be set
+ * @name: Atk name string
+ * @description: Atk description string
+ *
+ * This function sets up name and description
  * for a specified gtk widget.
  */
 void
-gedit_utils_set_atk_name_description (GtkWidget *widget, 
+gedit_utils_set_atk_name_description (GtkWidget   *widget, 
 				      const gchar *name,
 				      const gchar *description)
 {
@@ -304,24 +272,26 @@ gedit_utils_set_atk_name_description (GtkWidget *widget,
 	if (!(GTK_IS_ACCESSIBLE (aobj)))
 		return;
 
-	if(name)
+	if (name)
 		atk_object_set_name (aobj, name);
 
-	if(description)
+	if (description)
 		atk_object_set_description (aobj, description);
 }
 
 /**
- * gedit_set_atk__relation
- * @obj1,@obj2 : specified widgets.
- * @rel_type : the type of relation to set up.
- * Description : This function establishes atk relation
+ * gedit_set_atk_relation:
+ * @obj1: specified widget.
+ * @obj2: specified widget.
+ * @rel_type: the type of relation to set up.
+ *
+ * This function establishes atk relation
  * between 2 specified widgets.
  */
 void
-gedit_utils_set_atk_relation (GtkWidget *obj1, 
-			      GtkWidget *obj2, 
-			      AtkRelationType rel_type )
+gedit_utils_set_atk_relation (GtkWidget       *obj1, 
+			      GtkWidget       *obj2, 
+			      AtkRelationType  rel_type)
 {
 	AtkObject *atk_obj1, *atk_obj2;
 	AtkRelationSet *relation_set;
@@ -343,26 +313,6 @@ gedit_utils_set_atk_relation (GtkWidget *obj1,
 	g_object_unref (G_OBJECT (relation));
 }
 
-gboolean
-gedit_utils_uri_exists (const gchar* text_uri)
-{
-	GFile *gfile;
-	gboolean res;
-		
-	g_return_val_if_fail (text_uri != NULL, FALSE);
-	
-	gedit_debug_message (DEBUG_UTILS, "text_uri: %s", text_uri);
-
-	gfile = g_file_new_for_uri (text_uri);
-	res = g_file_query_exists (gfile, NULL);
-
-	g_object_unref (gfile);
-
-	gedit_debug_message (DEBUG_UTILS, res ? "TRUE" : "FALSE");
-
-	return res;
-}
-
 gchar *
 gedit_utils_escape_search_text (const gchar* text)
 {
@@ -451,21 +401,21 @@ gedit_utils_unescape_search_text (const gchar *text)
 			{
 				case 'n':
 					str = g_string_append (str, "\n");
-				break;
+					break;
 				case 'r':
 					str = g_string_append (str, "\r");
-				break;
+					break;
 				case 't':
 					str = g_string_append (str, "\t");
-				break;
+					break;
 				case '\\':
 					str = g_string_append (str, "\\");
 					drop_prev = TRUE;
-				break;
+					break;
 				default:
 					str = g_string_append (str, "\\");
 					str = g_string_append_len (str, cur, next - cur);
-				break;
+					break;
 			}
 		} 
 		else if (*cur != '\\') 
@@ -538,7 +488,7 @@ gedit_warning (GtkWindow *parent, const gchar *format, ...)
  * Doubles underscore to avoid spurious menu accels.
  */
 gchar * 
-gedit_utils_escape_underscores (const gchar* text,
+gedit_utils_escape_underscores (const gchar *text,
 				gssize       length)
 {
 	GString *str;
@@ -581,7 +531,7 @@ gedit_utils_escape_underscores (const gchar* text,
 static gchar *
 gedit_utils_str_truncate (const gchar *string,
 			  guint        truncate_length,
-			  gboolean middle)
+			  gboolean     middle)
 {
 	GString     *truncated;
 	guint        length;
@@ -602,14 +552,16 @@ gedit_utils_str_truncate (const gchar *string,
 	 * side)
 	 */
 	delimiter_length = g_utf8_strlen (delimiter, -1);
-	if (truncate_length < (delimiter_length + 2)) {
+	if (truncate_length < (delimiter_length + 2))
+	{
 		return g_strdup (string);
 	}
 
 	n_chars = g_utf8_strlen (string, length);
 
 	/* Make sure the string is not already small enough. */
-	if (n_chars <= truncate_length) {
+	if (n_chars <= truncate_length)
+	{
 		return g_strdup (string);
 	}
 
@@ -663,12 +615,14 @@ gedit_utils_make_valid_utf8 (const char *name)
 	remaining_bytes = strlen (name);
 
 	while (remaining_bytes != 0) {
-		if (g_utf8_validate (remainder, remaining_bytes, &invalid)) {
+		if (g_utf8_validate (remainder, remaining_bytes, &invalid))
+		{
 			break;
 		}
 		valid_bytes = invalid - remainder;
 
-		if (string == NULL) {
+		if (string == NULL)
+		{
 			string = g_string_sized_new (remaining_bytes);
 		}
 		g_string_append_len (string, remainder, valid_bytes);
@@ -679,7 +633,8 @@ gedit_utils_make_valid_utf8 (const char *name)
 		remainder = invalid + 1;
 	}
 
-	if (string == NULL) {
+	if (string == NULL)
+	{
 		return g_strdup (name);
 	}
 
@@ -690,7 +645,11 @@ gedit_utils_make_valid_utf8 (const char *name)
 	return g_string_free (string, FALSE);
 }
 
-/* Note that this function replace home dir with ~ */
+/**
+ * gedit_utils_uri_get_dirname:
+ *
+ * Note: this function replace home dir with ~
+ */
 gchar *
 gedit_utils_uri_get_dirname (const gchar *uri)
 {
@@ -746,30 +705,39 @@ gedit_utils_location_get_dirname_for_display (GFile *location)
 	if (mount != NULL)
 	{
 		gchar *mount_name;
-		gchar *path;
+		gchar *path = NULL;
+		gchar *dirname;
 
 		mount_name = g_mount_get_name (mount);
 		g_object_unref (mount);
 
-		/* obtain the "path" patrt of the uri */
-		if (gedit_utils_decode_uri (uri,
-					    NULL, NULL,
-					    NULL, NULL,
-					    &path))
-		{
-			gchar *dirname;
-			
-			dirname = gedit_utils_uri_get_dirname (path);
-			res = g_strdup_printf ("%s %s", mount_name, dirname);
+		/* obtain the "path" part of the uri */
+		gedit_utils_decode_uri (uri,
+					NULL, NULL,
+					NULL, NULL,
+					&path);
 
-			g_free (path);
-			g_free (dirname);
-			g_free (mount_name);
+		if (path == NULL)
+		{
+			dirname = gedit_utils_uri_get_dirname (uri);
 		}
 		else
+		{
+			dirname = gedit_utils_uri_get_dirname (path);
+		}
+		
+		if (dirname == NULL || strcmp (dirname, ".") == 0)
 		{
 			res = mount_name;
 		}
+		else
+		{
+			res = g_strdup_printf ("%s %s", mount_name, dirname);
+			g_free (mount_name);
+		}
+		
+		g_free (path);
+		g_free (dirname);
 	}
 	else
 	{
@@ -830,7 +798,9 @@ gedit_utils_replace_home_dir_with_tilde (const gchar *uri)
 /* the following two functions are courtesy of galeon */
 
 /**
- * gedit_utils_get_current_workspace: Get the current workspace
+ * gedit_utils_get_current_workspace:
+ *
+ * Get the current workspace
  *
  * Get the currently visible workspace for the #GdkScreen.
  *
@@ -879,7 +849,9 @@ gedit_utils_get_current_workspace (GdkScreen *screen)
 }
 
 /**
- * gedit_utils_get_window_workspace: Get the workspace the window is on
+ * gedit_utils_get_window_workspace:
+ *
+ * Get the workspace the window is on
  *
  * This function gets the workspace that the #GtkWindow is visible on,
  * it returns GEDIT_ALL_WORKSPACES if the window is sticky, or if
@@ -900,7 +872,7 @@ gedit_utils_get_window_workspace (GtkWindow *gtkwindow)
 	guint ret = GEDIT_ALL_WORKSPACES;
 
 	g_return_val_if_fail (GTK_IS_WINDOW (gtkwindow), 0);
-	g_return_val_if_fail (GTK_WIDGET_REALIZED (GTK_WIDGET (gtkwindow)), 0);
+	g_return_val_if_fail (gtk_widget_get_realized (GTK_WIDGET (gtkwindow)), 0);
 
 	window = gtk_widget_get_window (GTK_WIDGET (gtkwindow));
 	display = gdk_drawable_get_display (window);
@@ -928,7 +900,9 @@ gedit_utils_get_window_workspace (GtkWindow *gtkwindow)
 }
 
 /**
- * gedit_utils_get_current_viewport: Get the current viewport origin
+ * gedit_utils_get_current_viewport:
+ *
+ * Get the current viewport origin
  *
  * Get the currently visible viewport origin for the #GdkScreen.
  *
@@ -1007,11 +981,13 @@ has_valid_scheme (const gchar *uri)
 
 	p = uri;
 
-	if (!is_valid_scheme_character (*p)) {
+	if (!is_valid_scheme_character (*p))
+	{
 		return FALSE;
 	}
 
-	do {
+	do
+	{
 		p++;
 	} while (is_valid_scheme_character (*p));
 
@@ -1019,15 +995,24 @@ has_valid_scheme (const gchar *uri)
 }
 
 gboolean
-gedit_utils_is_valid_uri (const gchar *uri)
+gedit_utils_is_valid_location (GFile *location)
 {
 	const guchar *p;
+	gchar *uri;
+	gboolean is_valid;
 
-	if (uri == NULL)
+	if (location == NULL)
 		return FALSE;
 
+	uri = g_file_get_uri (location);
+
 	if (!has_valid_scheme (uri))
+	{
+		g_free (uri);
 		return FALSE;
+	}
+
+	is_valid = TRUE;
 
 	/* We expect to have a fully valid set of characters */
 	for (p = (const guchar *)uri; *p; p++) {
@@ -1035,20 +1020,31 @@ gedit_utils_is_valid_uri (const gchar *uri)
 		{
 			++p;
 			if (!g_ascii_isxdigit (*p))
-				return FALSE;
+			{
+				is_valid = FALSE;
+				break;
+			}
 
 			++p;		
 			if (!g_ascii_isxdigit (*p))
-				return FALSE;
+			{
+				is_valid = FALSE;
+				break;
+			}
 		}
 		else
 		{
 			if (*p <= 32 || *p >= 128)
-				return FALSE;
+			{
+				is_valid = FALSE;
+				break;
+			}
 		}
 	}
 
-	return TRUE;
+	g_free (uri);
+
+	return is_valid;
 }
 
 static GtkWidget *
@@ -1081,21 +1077,22 @@ handle_builder_error (const gchar *message, ...)
 	return label;
 }
 
+/* FIXME this is an issue for introspection */
 /**
  * gedit_utils_get_ui_objects:
  * @filename: the path to the gtk builder file
- * @root_objects: a NULL terminated list of root objects to load or NULL to
+ * @root_objects: a %NULL terminated list of root objects to load or NULL to
  *                load all objects
  * @error_widget: a pointer were a #GtkLabel
  * @object_name: the name of the first object
  * @...: a pointer were the first object is returned, followed by more
- *       name / object pairs and terminated by NULL.
+ *       name / object pairs and terminated by %NULL.
  *
  * This function gets the requested objects from a GtkBuilder ui file. In case
- * of error it returns FALSE and sets error_widget to a GtkLabel containing
+ * of error it returns %FALSE and sets error_widget to a GtkLabel containing
  * the error message to display.
  *
- * Returns FALSE if an error occurs, TRUE on success.
+ * Returns: %FALSE if an error occurs, %TRUE on success.
  */
 gboolean
 gedit_utils_get_ui_objects (const gchar  *filename,
@@ -1137,7 +1134,7 @@ gedit_utils_get_ui_objects (const gchar  *filename,
 
 	if (error != NULL)
 	{
-		*error_widget = handle_builder_error (_("Unable to open ui file %s. Error: %s"),
+		*error_widget = handle_builder_error (_("Unable to open UI file %s. Error: %s"),
 						      filename_markup,
 						      error->message);
 		g_error_free (error);
@@ -1214,60 +1211,40 @@ gedit_utils_make_canonical_uri_from_shell_arg (const gchar *str)
 	 */
 
 	gfile = g_file_new_for_commandline_arg (str);
-	uri = g_file_get_uri (gfile);
-	g_object_unref (gfile);
 
-	if (gedit_utils_is_valid_uri (uri))
+	if (gedit_utils_is_valid_location (gfile))
+	{
+		uri = g_file_get_uri (gfile);
+		g_object_unref (gfile);
 		return uri;
+	}
 	
-	g_free (uri);
+	g_object_unref (gfile);
 	return NULL;
 }
 
-/**
- * gedit_utils_file_has_parent:
- * @gfile: the GFile to check the parent for
- *
- * Return TRUE if the specified gfile has a parent (is not the root), FALSE
- * otherwise
- */
-gboolean
-gedit_utils_file_has_parent (GFile *gfile)
-{
-	GFile *parent;
-	gboolean ret;
-	
-	parent = g_file_get_parent (gfile);
-	ret = parent != NULL;
-	
-	if (parent)
-		g_object_unref (parent);
-	
-	return ret;
-}
-
 /**
  * gedit_utils_basename_for_display:
- * @uri: uri for which the basename should be displayed
+ * @location: location for which the basename should be displayed
  *
  * Return the basename of a file suitable for display to users.
  */
 gchar *
-gedit_utils_basename_for_display (gchar const *uri)
+gedit_utils_basename_for_display (GFile *location)
 {
 	gchar *name;
-	GFile *gfile;
 	gchar *hn;
+	gchar *uri;
 
-	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (G_IS_FILE (location), NULL);
 
-	gfile = g_file_new_for_uri (uri);
+	uri = g_file_get_uri (location);
 
 	/* First, try to query the display name, but only on local files */
-	if (g_file_has_uri_scheme (gfile, "file"))
+	if (gedit_utils_location_has_file_scheme (location))
 	{
 		GFileInfo *info;
-		info = g_file_query_info (gfile,
+		info = g_file_query_info (location,
 					  G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, 
 					  G_FILE_QUERY_INFO_NONE, 
 					  NULL, 
@@ -1285,12 +1262,13 @@ gedit_utils_basename_for_display (gchar const *uri)
 			 * g_filename_display_basename on the local path */
 			gchar *local_path;
 		
-			local_path = g_file_get_path (gfile);
+			local_path = g_file_get_path (location);
 			name = g_filename_display_basename (local_path);
 			g_free (local_path);
 		}
 	}
-	else if (gedit_utils_file_has_parent (gfile) || !gedit_utils_decode_uri (uri, NULL, NULL, &hn, NULL, NULL))
+	else if (g_file_has_parent (location, NULL) ||
+	          !gedit_utils_decode_uri (uri, NULL, NULL, &hn, NULL, NULL))
 	{
 		/* For remote files with a parent (so not just http://foo.com)
 		   or remote file for which the decoding of the host name fails,
@@ -1298,7 +1276,7 @@ gedit_utils_basename_for_display (gchar const *uri)
 		gchar *parse_name;
 		gchar *base;
 
-		parse_name = g_file_get_parse_name (gfile);
+		parse_name = g_file_get_parse_name (location);
 		base = g_filename_display_basename (parse_name);
 		name = g_uri_unescape_string (base, NULL);
 		
@@ -1311,10 +1289,14 @@ gedit_utils_basename_for_display (gchar const *uri)
 		gchar *hn_utf8;
 
 		if  (hn != NULL)
+		{
 			hn_utf8 = gedit_utils_make_valid_utf8 (hn);
+		}
 		else
+		{
 			/* we should never get here */
 			hn_utf8 = g_strdup ("?");
+		}
 
 		/* Translators: '/ on <remote-share>' */
 		name = g_strdup_printf (_("/ on %s"), hn_utf8);
@@ -1323,14 +1305,14 @@ gedit_utils_basename_for_display (gchar const *uri)
 		g_free (hn);
 	}
 
-	g_object_unref (gfile);
+	g_free (uri);
 
 	return name;
 }
 
 /**
  * gedit_utils_uri_for_display:
- * @uri: uri to be displayed.
+ * @location: location to be displayed.
  *
  * Filter, modify, unescape and change @uri to make it appropriate
  * for display to users.
@@ -1340,26 +1322,18 @@ gedit_utils_basename_for_display (gchar const *uri)
  * Return value: a string which represents @uri and can be displayed.
  */
 gchar *
-gedit_utils_uri_for_display (const gchar *uri)
+gedit_utils_uri_for_display (GFile *location)
 {
-	GFile *gfile;
-	gchar *parse_name;
-
-	gfile = g_file_new_for_uri (uri);
-	parse_name = g_file_get_parse_name (gfile);
-	g_object_unref (gfile);
-
-	return parse_name;
+	return g_file_get_parse_name (location);
 }
 
 /**
  * gedit_utils_drop_get_uris:
  * @selection_data: the #GtkSelectionData from drag_data_received
- * @info: the info from drag_data_received
  *
  * Create a list of valid uri's from a uri-list drop.
  * 
- * Return value: a string array which will hold the uris or NULL if there 
+ * Return value: a string array which will hold the uris or %NULL if there 
  *		 were no valid uris. g_strfreev should be used when the 
  *		 string array is no longer used
  */
@@ -1404,26 +1378,27 @@ null_ptr (gchar **ptr)
 /**
  * gedit_utils_decode_uri:
  * @uri: the uri to decode
- * @scheme: return value pointer for the uri's scheme (e.g. http, sftp, ...)
- * @user: return value pointer for the uri user info
- * @port: return value pointer for the uri port
- * @host: return value pointer for the uri host
- * @path: return value pointer for the uri path
+ * @scheme: (allow-none): return value pointer for the uri's
+ * scheme (e.g. http, sftp, ...), or %NULL
+ * @user: (allow-none): return value pointer for the uri user info, or %NULL
+ * @port: (allow-none): return value pointer for the uri port, or %NULL
+ * @host: (allow-none): return value pointer for the uri host, or %NULL
+ * @path: (allow-none): return value pointer for the uri path, or %NULL
  *
  * Parse and break an uri apart in its individual components like the uri
  * scheme, user info, port, host and path. The return value pointer can be
- * NULL to ignore certain parts of the uri. If the function returns TRUE, then
+ * %NULL to ignore certain parts of the uri. If the function returns %TRUE, then
  * all return value pointers should be freed using g_free
  * 
- * Return value: TRUE if the uri could be properly decoded, FALSE otherwise.
+ * Return value: %TRUE if the uri could be properly decoded, %FALSE otherwise.
  */
 gboolean
-gedit_utils_decode_uri (const gchar *uri,
-			gchar **scheme,
-			gchar **user,
-			gchar **host,
-			gchar **port,
-			gchar **path
+gedit_utils_decode_uri (const gchar  *uri,
+			gchar       **scheme,
+			gchar       **user,
+			gchar       **host,
+			gchar       **port,
+			gchar       **path
 )
 {
 	/* Largely copied from glib/gio/gdummyfile.c:_g_decode_uri. This 
@@ -1464,7 +1439,9 @@ gedit_utils_decode_uri (const gchar *uri,
 		      c == '+' ||
 		      c == '-' ||
 		      c == '.'))
+		{
 			return FALSE;
+		}
 	}
 	
 	if (scheme)
@@ -1473,7 +1450,9 @@ gedit_utils_decode_uri (const gchar *uri,
 		out = *scheme;
 	
 		for (in = uri; in < p - 1; in++)
+		{
 			*out++ = g_ascii_tolower (*in);
+		}
 			
 		*out = '\0';
 	}
@@ -1519,7 +1498,9 @@ gedit_utils_decode_uri (const gchar *uri,
 			host_start = userinfo_end + 1;
 		}
 		else
+		{
 			host_start = authority_start;
+		}
 
 		port_start = memchr (host_start, ':', authority_end - host_start);
 
@@ -1531,7 +1512,9 @@ gedit_utils_decode_uri (const gchar *uri,
 				*port = g_strndup (port_start, authority_end - port_start);
 		}
 		else
+		{
 			host_end = authority_end;
+		}
 
 		if (host)
 			*host = g_strndup (host_start, host_end - host_start);
@@ -1544,3 +1527,39 @@ gedit_utils_decode_uri (const gchar *uri,
 	
 	return TRUE;
 }
+
+gboolean
+gedit_utils_can_read_from_stdin (void)
+{
+#ifdef G_OS_UNIX
+	if (!isatty (STDIN_FILENO))
+	{
+		struct stat sbuffer;
+
+		if (fstat (STDIN_FILENO, &sbuffer) == 0 &&
+		    (S_ISREG (sbuffer.st_mode) || S_ISFIFO (sbuffer.st_mode) || S_ISLNK (sbuffer.st_mode)))
+		{
+			return TRUE;
+		}
+	}
+#endif
+	return FALSE;
+}
+
+GeditDocumentCompressionType
+gedit_utils_get_compression_type_from_content_type (const gchar *content_type)
+{
+	if (content_type == NULL)
+	{
+		return GEDIT_DOCUMENT_COMPRESSION_TYPE_NONE;
+	}
+
+	if (g_content_type_is_a (content_type, "application/x-gzip"))
+	{
+		return GEDIT_DOCUMENT_COMPRESSION_TYPE_GZIP;
+	}
+
+	return GEDIT_DOCUMENT_COMPRESSION_TYPE_NONE;
+}
+
+/* ex:ts=8:noet: */
