@@ -38,18 +38,19 @@
 #include <stdlib.h>
 
 #include <gdk/gdkkeysyms.h>
+#include <libpeas/peas-extension-set.h>
 
 #include <glib/gi18n.h>
 
 #include "gedit-view.h"
+#include "gedit-view-activatable.h"
+#include "gedit-plugins-engine.h"
 #include "gedit-debug.h"
 #include "gedit-marshal.h"
 #include "gedit-utils.h"
-#include "gseal-gtk-compat.h"
 #include "gedit-settings.h"
 #include "gedit-app.h"
 
-
 #define GEDIT_VIEW_SCROLL_MARGIN 0.02
 #define GEDIT_VIEW_SEARCH_DIALOG_TIMEOUT (30*1000) /* 30 seconds */
 
@@ -98,6 +99,8 @@ struct _GeditViewPrivate
 	gboolean     disable_popdown;
 	
 	GtkTextBuffer *current_buffer;
+
+	PeasExtensionSet *extensions;
 };
 
 /* The search entry completion is shared among all the views */
@@ -306,6 +309,24 @@ current_buffer_removed (GeditView *view)
 	}
 }
 
+static void
+extension_added (PeasExtensionSet *extensions,
+		 PeasPluginInfo   *info,
+		 PeasExtension    *exten,
+		 GeditView        *view)
+{
+	peas_extension_call (exten, "activate", view);
+}
+
+static void
+extension_removed (PeasExtensionSet *extensions,
+		   PeasPluginInfo   *info,
+		   PeasExtension    *exten,
+		   GeditView        *view)
+{
+	peas_extension_call (exten, "deactivate", view);
+}
+
 static void
 on_notify_buffer_cb (GeditView  *view,
 		     GParamSpec *arg1,
@@ -332,6 +353,11 @@ on_notify_buffer_cb (GeditView  *view,
 			  "search_highlight_updated",
 			  G_CALLBACK (search_highlight_updated_cb),
 			  view);
+
+	/* We only activate the extensions when the right buffer is set,
+	 * because most plugins will expect this behaviour, and we won't
+	 * change the buffer later anyway. */
+	peas_extension_set_call (view->priv->extensions, "activate", view);
 }
 
 static void 
@@ -396,10 +422,11 @@ gedit_view_init (GeditView *view)
 	hl_current_line = g_settings_get_boolean (view->priv->editor_settings,
 						  GEDIT_SETTINGS_HIGHLIGHT_CURRENT_LINE);
 
-	wrap_mode = gedit_settings_get_wrap_mode (view->priv->editor_settings,
-						  GEDIT_SETTINGS_WRAP_MODE);
+	wrap_mode = g_settings_get_enum (view->priv->editor_settings,
+					 GEDIT_SETTINGS_WRAP_MODE);
 	
-	smart_home_end = gedit_settings_get_smart_home_end (GEDIT_SETTINGS (gs));
+	smart_home_end = g_settings_get_enum (view->priv->editor_settings,
+					      GEDIT_SETTINGS_SMART_HOME_END);
 
 	g_object_set (G_OBJECT (view), 
 		      "wrap_mode", wrap_mode,
@@ -423,11 +450,23 @@ gedit_view_init (GeditView *view)
 	if (tl != NULL)
 		gtk_target_list_add_uri_targets (tl, TARGET_URI_LIST);
 		
+	view->priv->extensions = peas_extension_set_new (PEAS_ENGINE (gedit_plugins_engine_get_default ()),
+							 GEDIT_TYPE_VIEW_ACTIVATABLE);
+	g_signal_connect (view->priv->extensions,
+			  "extension-added",
+			  G_CALLBACK (extension_added),
+			  view);
+	g_signal_connect (view->priv->extensions,
+			  "extension-removed",
+			  G_CALLBACK (extension_removed),
+			  view);
+
 	/* Act on buffer change */
 	g_signal_connect (view, 
 			  "notify::buffer", 
 			  G_CALLBACK (on_notify_buffer_cb),
 			  NULL);
+
 }
 
 static void
@@ -437,6 +476,15 @@ gedit_view_destroy (GtkObject *object)
 
 	view = GEDIT_VIEW (object);
 
+	if (view->priv->extensions != NULL)
+	{
+		peas_extension_set_call (view->priv->extensions,
+					 "deactivate",
+					 view);
+		g_object_unref (view->priv->extensions);
+		view->priv->extensions = NULL;
+	}
+
 	if (view->priv->search_window != NULL)
 	{
 		gtk_widget_destroy (view->priv->search_window);
@@ -500,7 +548,7 @@ gedit_view_focus_out (GtkWidget *widget, GdkEventFocus *event)
  * @doc: a #GeditDocument
  * 
  * Creates a new #GeditView object displaying the @doc document. 
- * @doc cannot be NULL.
+ * @doc cannot be %NULL.
  *
  * Return value: a new #GeditView
  **/
@@ -686,6 +734,7 @@ gedit_view_scroll_to_cursor (GeditView *view)
 				      0.0);
 }
 
+/* FIXME this is an issue for introspection */
 /**
  * gedit_view_set_font:
  * @view: a #GeditView
@@ -952,26 +1001,12 @@ send_focus_change (GtkWidget *widget,
 	GdkEvent *fevent = gdk_event_new (GDK_FOCUS_CHANGE);
 
 	g_object_ref (widget);
-#if !GTK_CHECK_VERSION (2, 21, 0)
-	if (in)
-		GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-	else
-		GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-#endif
 
 	fevent->focus_change.type = GDK_FOCUS_CHANGE;
 	fevent->focus_change.window = g_object_ref (gtk_widget_get_window (widget));
 	fevent->focus_change.in = in;
 
-#if !GTK_CHECK_VERSION (2, 21, 0)
-	gtk_widget_event (widget, fevent);
-  
-	g_object_notify (G_OBJECT (widget), "has-focus");
-
-	g_object_unref (widget);
-#else
 	gtk_widget_send_focus_change (widget, fevent);
-#endif
 
 	gdk_event_free (fevent);
 }
@@ -1492,12 +1527,12 @@ ensure_search_window (GeditView *view)
 
 	if (view->priv->search_window != NULL)
 	{
-		if (gtk_window_get_group (GTK_WINDOW (toplevel)))
+		if (gtk_window_has_group (GTK_WINDOW (toplevel)))
 		{
 			gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (toplevel)),
 						     GTK_WINDOW (view->priv->search_window));
 		}
-		else if (gtk_window_get_group (GTK_WINDOW (view->priv->search_window)))
+		else if (gtk_window_has_group (GTK_WINDOW (view->priv->search_window)))
 		{
 			gtk_window_group_remove_window (gtk_window_get_group (GTK_WINDOW (view->priv->search_window)),
 							GTK_WINDOW (view->priv->search_window));
@@ -1510,7 +1545,7 @@ ensure_search_window (GeditView *view)
    
 	view->priv->search_window = gtk_window_new (GTK_WINDOW_POPUP);
 
-	if (gtk_window_get_group (GTK_WINDOW (toplevel)))
+	if (gtk_window_has_group (GTK_WINDOW (toplevel)))
 	{
 		gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (toplevel)),
 					     GTK_WINDOW (view->priv->search_window));
@@ -1957,7 +1992,9 @@ gedit_view_drag_motion (GtkWidget      *widget,
 	/* If this is a URL, deal with it here */
 	if (drag_get_uri_target (widget, context) != GDK_NONE) 
 	{
-		gdk_drag_status (context, context->suggested_action, timestamp);
+		gdk_drag_status (context,
+				 gdk_drag_context_get_suggested_action (context),
+				 timestamp);
 		result = TRUE;
 	}
 
